@@ -1,21 +1,24 @@
 import chromadb
 import chromadb.utils.embedding_functions as embedding_functions
+from chromadb.config import Settings
 import logging
 import os
 import uuid
-from typing import List, Optional, Dict, Any
-from datetime import datetime, timedelta
+import re
+from typing import List, Optional, Dict, Any, Tuple
+from datetime import datetime, timedelta, timezone
 import dotenv
-from langfuse import observe
 from pathlib import Path
 
 from .base_memory import MemoryStorageBase
-from .llm.llm_service import MemoryLLMService
+from .utils.llm_service import MemoryLLMService
+from .utils.memory_config import TRIGGER_KEYWORDS, MAX_SCAN_LENGTH
 
 dotenv.load_dotenv(override=True)
 
 class MemoryStorageLocal(MemoryStorageBase):
-    """Local memory implementation using ChromaDB with LLM-powered memory extraction.
+    """
+    Local memory storage using ChromaDB with LLM-based memory extraction.
     
     Args:
         path: Path to ChromaDB storage directory. Defaults to ~/.xagent/chroma
@@ -35,7 +38,7 @@ class MemoryStorageLocal(MemoryStorageBase):
         # Use default path if none provided
         if path is None:
             path = os.path.expanduser('~/.xagent/chroma')
-            logging.info("No path provided, using default path: %s", path)
+            self.logger.info("No path provided, using default path: %s", path)
         
         # Ensure the directory exists
         Path(path).mkdir(parents=True, exist_ok=True)
@@ -49,7 +52,7 @@ class MemoryStorageLocal(MemoryStorageBase):
         )
         
         # Initialize ChromaDB client and collection
-        self.chroma_client = chromadb.PersistentClient(path=path)
+        self.chroma_client = chromadb.PersistentClient(path=path,settings=Settings(anonymized_telemetry=False))
         self.collection = self.chroma_client.get_or_create_collection(
             name=collection_name,
             embedding_function=self.openai_ef
@@ -62,6 +65,9 @@ class MemoryStorageLocal(MemoryStorageBase):
         self.memory_threshold = memory_threshold  # Store to long-term memory when reaching this many messages
         self.keep_recent = keep_recent  # Keep this many most recent messages after storage
         
+        # 编译正则表达式模式，提高匹配性能
+        self._compiled_patterns = self._compile_keyword_patterns()
+        
         self.logger.info("LocalMemory initialized with collection: %s", collection_name)
     
     async def add(self,
@@ -80,24 +86,6 @@ class MemoryStorageLocal(MemoryStorageBase):
             self.logger.debug("No messages provided for user %s", user_id)
             return
 
-        # 关键词列表（中英文）
-        trigger_keywords = [
-            # 中文 - 直接指令
-            "记住", "记得", "记一下", "记下来", "记录一下", "帮我记", 
-            "请记住", "请帮我记住", "保存一下", "存下来", "备忘一下", "写下来",
-
-            # 中文 - 偏好/事实
-            "我喜欢", "我爱", "我讨厌", "我最喜欢", "我常", "我是",
-
-            # 英文 - 直接指令
-            "remember this", "remember that", "please remember", "help me remember",
-            "note this", "note this down", "make a note", "save this", 
-            "keep this", "log this", "store this", "write this down",
-
-            # 英文 - 偏好/事实
-            "I like", "I love", "I hate", "I prefer", "my favorite is", "I usually", "I am"
-        ]
-
         if user_id not in self._user_messages:
             self._user_messages[user_id] = []
 
@@ -110,19 +98,30 @@ class MemoryStorageLocal(MemoryStorageBase):
 
         # 只监测 role 为 'user' 的消息是否有关键词触发，逆序遍历（从最新一条开始）
         keyword_triggered = False
+        trigger_tier = ""
+        
         for msg in reversed(messages):
             if msg.get("role", "") != "user":
                 continue
+            
             content = msg.get("content", "")
-            if any(kw.lower() in content.lower() for kw in trigger_keywords):
+            is_triggered, tier = self._check_keyword_trigger(content)
+            
+            if is_triggered:
                 keyword_triggered = True
-                self.logger.info("Keyword trigger detected in user message for user %s: %s", user_id, content)
+                trigger_tier = tier
+                self.logger.info(
+                    "Keyword trigger detected for user %s - Tier: %s, Content: %s", 
+                    user_id, tier, content[:100] + "..." if len(content) > 100 else content
+                )
                 break
 
         # Check if threshold is reached or keyword is triggered
         if message_count >= self.memory_threshold or keyword_triggered:
-            self.logger.info("Triggering memory storage for user %s (reason: %s)", 
-                            user_id, "keyword" if keyword_triggered else "threshold")
+            self.logger.info("Triggering memory storage for user %s (reason: %s%s)", 
+                            user_id, 
+                            "keyword" if keyword_triggered else "threshold",
+                            f" - {trigger_tier}" if trigger_tier else "")
             try:
                 # Convert messages to conversation format for storage
                 conversation_content = self._format_messages_for_storage(self._user_messages[user_id])
@@ -147,31 +146,29 @@ class MemoryStorageLocal(MemoryStorageBase):
               content: str) -> str:
         """Store memory content with LLM-based extraction and return memory ID."""
         self.logger.info("Storing memory for user: %s, content length: %d", user_id, len(content))
-        
         # Extract structured memories from content
         extracted_memories = await self.llm_service.extract_memories_from_content(content)
-        now = datetime.now()
-        
+
         # Prepare data for batch storage
         documents = []
         metadatas = []
-        
+
         # Process extracted memories
         for memory_piece in extracted_memories.memories:
             documents.append(memory_piece.content)
-            metadatas.append(self._create_base_metadata(user_id, memory_piece.type.value, None, now))
+            metadatas.append(self._create_base_metadata(user_id, memory_piece.type.value))
 
         # If no memories were extracted, do not store anything
         if not documents:
             self.logger.debug("No structured memories extracted, nothing stored for user %s", user_id)
             return ""  # Return empty string when no memories extracted
-        
+
         # Batch store all memories
         memory_ids = self._batch_store_memories(documents, metadatas)
-        
+
         log_msg = f"Stored {len(memory_ids)} {'extracted memory pieces' if len(memory_ids) > 1 else 'memory piece'} for user {user_id}"
         self.logger.debug(log_msg)
-        
+
         return memory_ids[0] if len(memory_ids) == 1 else str(memory_ids)
     
     async def retrieve(self, 
@@ -203,16 +200,22 @@ class MemoryStorageLocal(MemoryStorageBase):
             # Use ChromaDB's batch query capability
             results = self.collection.query(
                 query_texts=query_texts,
-                n_results=min(limit, 20), 
+                n_results=min(limit * 2, 20), 
                 where={"user_id": user_id},
                 include=["documents", "metadatas", "distances"]
             )
 
             if preprocessed.keywords:
+                # Build keyword query - use $or only if multiple keywords, otherwise use simple $contains
+                if len(preprocessed.keywords) > 1:
+                    keyword_query = {"$or": [{"$contains": kw} for kw in preprocessed.keywords]}
+                else:
+                    keyword_query = {"$contains": preprocessed.keywords[0]}
+                    
                 results_keyword_match = self.collection.get(
                     limit=min(limit, 20), 
                     where={"user_id": user_id},
-                    where_document={"$or": [{"$contains": kw} for kw in preprocessed.keywords] } if preprocessed.keywords else None,
+                    where_document=keyword_query,
                     include=["documents", "metadatas"]
                 )
             else:
@@ -282,7 +285,21 @@ class MemoryStorageLocal(MemoryStorageBase):
         except Exception as e:
             self.logger.error("Error in batch query search: %s", str(e))
             # Fallback to single query search
-            return await self._fallback_retrieve(preprocessed.original_query, user_id, limit)
+            try:
+                results = self.collection.query(
+                    query_texts=[preprocessed.original_query],
+                    n_results=limit,
+                    where={"user_id": user_id},
+                    include=["documents", "metadatas"]
+                )
+                
+                memories = self._format_memory_results(results)
+                self.logger.debug("Fallback: Retrieved %d memories for user %s", len(memories), user_id)
+                return memories
+                
+            except Exception as fallback_e:
+                self.logger.error("Fallback query also failed: %s", str(fallback_e))
+                return []
 
     async def clear(self, user_id: str) -> None:
         """Clear all memories for a user."""
@@ -307,8 +324,7 @@ class MemoryStorageLocal(MemoryStorageBase):
         
         for piece in meta_memory.contents:
             documents.append(piece.content)
-            meta = self._create_base_metadata(user_id, piece.type.value, {}, datetime.now())
-            metadatas.append(meta)
+            metadatas.append(self._create_base_metadata(user_id, piece.type.value))
         
         # Batch store all meta contents
         memory_ids = self._batch_store_memories(documents, metadatas)
@@ -317,24 +333,50 @@ class MemoryStorageLocal(MemoryStorageBase):
         return memory_ids
 
 
-    async def _fallback_retrieve(self, query: str, user_id: str, limit: int) -> List[Dict[str, Any]]:
-        """Fallback retrieval method using single query."""
-        try:
-            results = self.collection.query(
-                query_texts=[query],
-                n_results=limit,
-                where={"user_id": user_id},
-                include=["documents", "metadatas"]
-            )
+    def _compile_keyword_patterns(self) -> List[re.Pattern]:
+        """编译关键字正则表达式模式，每个层级合并为一个 alternation 正则"""
+        compiled_patterns = []
+        
+        for tier_patterns in TRIGGER_KEYWORDS:
+            # 将同一层级的所有模式合并为一个 alternation 正则
+            combined_pattern = '(?:' + '|'.join(tier_patterns) + ')'
+            compiled_patterns.append(re.compile(combined_pattern, re.IGNORECASE | re.UNICODE))
+        
+        return compiled_patterns
+    
+    def _check_keyword_trigger(self, content: str) -> Tuple[bool, str]:
+        """
+        检查内容是否包含触发关键字，按层级优先检测
+        使用优化的 alternation 正则，按层级索引顺序早退检测
+        
+        Args:
+            content: 要检查的文本内容
             
-            memories = self._format_memory_results(results)
-            self.logger.debug("Fallback: Retrieved %d memories for user %s", len(memories), user_id)
-            return memories
-            
-        except Exception as fallback_e:
-            self.logger.error("Fallback query also failed: %s", str(fallback_e))
-            return []
+        Returns:
+            Tuple[bool, str]: (是否触发, 匹配的层级索引)
+        """
+        if not content:
+            return False, ""
+        
+        # 应用扫描长度限制，避免极长消息的性能抖动
+        if len(content) > MAX_SCAN_LENGTH:
+            content = content[:MAX_SCAN_LENGTH]
+            self.logger.debug("Content truncated to %d characters for keyword scanning", MAX_SCAN_LENGTH)
 
+        # 预处理文本：去除多余空格，统一格式
+        cleaned_content = re.sub(r'\s+', ' ', content.strip())
+        
+        # 按层级优先级检查，早退机制
+        for tier_index, pattern in enumerate(self._compiled_patterns):
+            if pattern.search(cleaned_content):
+                tier_name = f"tier{tier_index + 1}"
+                self.logger.debug(
+                    "Keyword trigger detected - Tier: %s, Content: %s",
+                    tier_name, cleaned_content[:100] + "..." if len(cleaned_content) > 100 else cleaned_content
+                )
+                return True, tier_name
+        
+        return False, ""
 
     def _format_messages_for_storage(self, messages: List[Dict[str, Any]]) -> str:
         """
@@ -365,20 +407,16 @@ class MemoryStorageLocal(MemoryStorageBase):
         return "\n\n".join(conversation_lines)
 
 
-    def _create_base_metadata(self, user_id: str, memory_type: str, 
-                             additional_metadata: Optional[Dict[str, Any]] = None,
-                             timestamp: Optional[datetime] = None) -> Dict[str, Any]:
+    def _create_base_metadata(self, user_id: str, memory_type: str) -> Dict[str, Any]:
         """Create base metadata for memory storage."""
-        now = timestamp or datetime.now()
-        meta = {
+        now = datetime.now(timezone.utc)
+        
+        return {
             "user_id": user_id,
             "created_at": now.isoformat(),
             "created_timestamp": now.timestamp(),
             "memory_type": memory_type,
         }
-        if additional_metadata:
-            meta.update(additional_metadata)
-        return meta
     
     def _batch_store_memories(self, documents: List[str], metadatas: List[Dict[str, Any]], 
                              ids: Optional[List[str]] = None) -> List[str]:
@@ -398,11 +436,18 @@ class MemoryStorageLocal(MemoryStorageBase):
         """Format ChromaDB results into standard memory format."""
         memories = []
         if results.get("documents"):
-            for doc_id, content, metadata in zip(
-                results["ids"], 
-                results["documents"], 
-                results["metadatas"]
-            ):
+            # Handle both single query and batch query results
+            ids = results["ids"]
+            documents = results["documents"] 
+            metadatas = results["metadatas"]
+            
+            # Flatten if nested (batch query results)
+            if isinstance(ids[0], list):
+                ids = [item for sublist in ids for item in sublist]
+                documents = [item for sublist in documents for item in sublist]
+                metadatas = [item for sublist in metadatas for item in sublist]
+            
+            for doc_id, content, metadata in zip(ids, documents, metadatas):
                 # Remove created_timestamp and user_id from metadata
                 filtered_metadata = {k: v for k, v in metadata.items() 
                                    if k not in ["created_timestamp", "user_id"]}
@@ -416,10 +461,10 @@ class MemoryStorageLocal(MemoryStorageBase):
         """Get all memories for a user created within the last N days."""
         self.logger.info("Retrieving last %d day(s) memories for user: %s", days, user_id)
         
-        # Calculate time range
-        start_date = datetime.now() - timedelta(days=days-1)
-        start_timestamp = start_date.replace(hour=0, minute=0, second=0, microsecond=0).timestamp()
-        end_timestamp = datetime.now().timestamp()
+        # Calculate timestamp range - much simpler approach
+        now = datetime.now(timezone.utc)
+        start_timestamp = (now - timedelta(days=days)).timestamp()
+        end_timestamp = now.timestamp()
         
         try:
             results = self.collection.get(
